"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.add = void 0;
const inquirer_1 = __importDefault(require("inquirer"));
const fs_1 = __importDefault(require("fs"));
// this is compatible with node@12+
const fs = fs_1.default.promises;
const path_1 = require("path");
const files_1 = require("@covector/files");
const add = function* ({ cwd = process.cwd(), changeFolder = ".changes", yes, }) {
    const config = yield (0, files_1.configFile)({ cwd });
    let packageBumps = {};
    const answers = yield inquirer_1.default.prompt([
        {
            type: "checkbox",
            message: "Select packages which need a version bump.",
            name: "packages",
            choices: Object.keys(config.packages).map((pkg) => ({ name: pkg })),
            validate(answer) {
                if (answer.length < 1) {
                    return "You must choose at least one package.";
                }
                return true;
            },
        },
    ]);
    for (let pkg of answers.packages) {
        packageBumps[pkg] = yield inquirer_1.default.prompt({
            type: "list",
            name: "bump",
            message: `bump ${pkg} with?`,
            choices: ["patch", "minor", "major"],
        });
    }
    const summary = yield inquirer_1.default.prompt({
        type: "input",
        name: "input",
        message: `Please summarize the changes that occurred.`,
        validate(answer) {
            if (answer.length < 1) {
                return "You must enter a summary.";
            }
            return true;
        },
    });
    const file = yield inquirer_1.default.prompt({
        type: "input",
        name: "name",
        message: `Please name the change file.`,
        validate(answer) {
            if (answer.length < 1) {
                return "You must enter a file name.";
            }
            return true;
        },
        filter: (input) => {
            if (input.endsWith(".md")) {
                return input;
            }
            else {
                return `${input}.md`;
            }
        },
    });
    const frontmatter = `---
${answers.packages
        //@ts-expect-error
        .map((pkg) => `"${pkg}": ${packageBumps[pkg].bump}`)
        .join("\n")}
---\n\n`;
    const content = `${frontmatter}${summary.input}\n`;
    yield fs.writeFile((0, path_1.join)(cwd, changeFolder, `${file.name}`), content);
    return "complete";
};
exports.add = add;
