"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.arbitrary = void 0;
const command_1 = require("@covector/command");
const files_1 = require("@covector/files");
const assemble_1 = require("@covector/assemble");
const changelog_1 = require("@covector/changelog");
function* arbitrary({ command, dryRun = false, cwd = process.cwd(), filterPackages = [], modifyConfig = (c) => __awaiter(this, void 0, void 0, function* () { return c; }), }) {
    const config = yield modifyConfig(yield (0, files_1.configFile)({ cwd }));
    const pre = yield (0, files_1.readPreFile)({ cwd, changeFolder: config.changeFolder });
    const changesPaths = yield (0, files_1.changeFiles)({
        cwd,
        changeFolder: config.changeFolder,
    });
    const changeFilesLoaded = yield (0, files_1.loadChangeFiles)({
        cwd,
        paths: changesPaths,
    });
    const assembledChanges = yield (0, assemble_1.assemble)({
        cwd,
        files: changeFilesLoaded,
        config,
        preMode: { on: !!pre, prevFiles: !pre ? [] : pre.changes },
    });
    yield (0, command_1.raceTime)({ t: config.timeout });
    const changelogs = yield (0, changelog_1.pullLastChangelog)({
        config,
        cwd,
    });
    const { commands, pipeTemplate, } = yield (0, assemble_1.mergeIntoConfig)({
        assembledChanges,
        config,
        command,
        cwd,
        dryRun,
        filterPackages,
        changelogs,
    });
    if (dryRun) {
        console.log("==== commands ready to run ===");
        console.log(commands);
    }
    if (commands.length === 0) {
        console.log(`No commands configured to run on [${command}].`);
        return {
            response: `No commands configured to run on [${command}].`,
        };
    }
    const commandsToRun = yield (0, command_1.confirmCommandsToRun)({
        cwd,
        commands,
        command,
    });
    let pkgCommandsRan = commands.reduce((pkgs, pkg) => {
        pkgs[pkg.pkg] = {
            precommand: false,
            command: false,
            postcommand: false,
            pkg,
        };
        return pkgs;
    }, {});
    pkgCommandsRan = yield (0, changelog_1.pipeChangelogToCommands)({
        changelogs,
        pkgCommandsRan,
    });
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands: commandsToRun,
        commandPrefix: "pre",
        command,
        pkgCommandsRan,
        dryRun,
    });
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands: commandsToRun,
        command,
        pkgCommandsRan,
        dryRun,
    });
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands: commandsToRun,
        commandPrefix: "post",
        command,
        pkgCommandsRan,
        dryRun,
    });
    if (dryRun) {
        console.log("==== result ===");
        console.log(pkgCommandsRan);
    }
    return { commandsRan: pkgCommandsRan, pipeTemplate };
}
exports.arbitrary = arbitrary;
