"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cli = void 0;
const yargs_1 = __importDefault(require("yargs"));
const run_1 = require("./run");
function* cli(argv) {
    const { command, directory, yes, dryRun } = parseOptions(argv);
    return yield (0, run_1.covector)({
        command,
        changeFolder: directory,
        yes,
        dryRun,
    });
}
exports.cli = cli;
function parseOptions(argv) {
    let rawOptions = yargs_1.default
        .scriptName("covector")
        .command("init", "initialize covector in your repo", function (yargs) {
        return yargs
            .option("yes", {
            alias: "y",
            describe: "skip all questions using the default response for each",
            default: false,
        })
            .boolean("yes")
            .option("directory", {
            alias: "d",
            describe: "specify a custom change folder",
            default: ".changes",
            hidden: true,
        });
    })
        .command(["status", "*"], "run status command")
        .command("config", "output current config")
        .command("version", "run version command")
        .command("publish", "run publish command")
        .options({
        "dry-run": {
            type: "boolean",
            default: false,
            describe: "run a command that shows the expected command without executing",
        },
    })
        .demandCommand(1)
        .help()
        .epilogue("For more information on covector, see: https://www.github.com/jbolda/covector")
        .parse(argv);
    return {
        command: String(rawOptions._[0]),
        dryRun: rawOptions["dry-run"],
        yes: rawOptions.yes,
        directory: rawOptions.directory,
    };
}
