"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.preview = void 0;
const command_1 = require("@covector/command");
const files_1 = require("@covector/files");
const assemble_1 = require("@covector/assemble");
const apply_1 = require("@covector/apply");
function* preview({ command, dryRun = false, cwd = process.cwd(), filterPackages = [], modifyConfig = (c) => __awaiter(this, void 0, void 0, function* () { return c; }), previewVersion = "", branchTag = "", }) {
    const config = yield modifyConfig(yield (0, files_1.configFile)({ cwd }));
    const pre = yield (0, files_1.readPreFile)({ cwd, changeFolder: config.changeFolder });
    const prereleaseIdentifier = !pre ? null : pre.tag;
    const changesPaths = yield (0, files_1.changeFiles)({
        cwd,
        changeFolder: config.changeFolder,
    });
    const changeFilesLoaded = yield (0, files_1.loadChangeFiles)({
        cwd,
        paths: changesPaths,
    });
    const assembledChanges = yield (0, assemble_1.assemble)({
        cwd,
        files: changeFilesLoaded,
        config,
        preMode: { on: !!pre, prevFiles: !pre ? [] : pre.changes },
    });
    const allPackages = yield (0, files_1.readAllPkgFiles)({
        config,
        cwd,
    });
    yield (0, command_1.raceTime)({ t: config.timeout });
    const versionChanges = (0, apply_1.changesConsideringParents)({
        assembledChanges,
        config,
        allPackages,
        prereleaseIdentifier,
    });
    const { commands: versionCommands } = yield (0, assemble_1.mergeChangesToConfig)({
        assembledChanges: versionChanges,
        config,
        command: "version",
        dryRun,
        filterPackages,
        cwd,
    });
    let pkgCommandsRan = Object.keys(config.packages).reduce((pkgs, pkg) => {
        pkgs[pkg] = {
            precommand: false,
            command: false,
            postcommand: false,
            applied: false,
        };
        return pkgs;
    }, {});
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands: versionCommands,
        commandPrefix: "pre",
        command: "version",
        pkgCommandsRan,
        dryRun,
    });
    const applied = yield (0, apply_1.apply)({
        //@ts-expect-error
        commands: versionCommands,
        allPackages,
        cwd,
        bump: !dryRun,
        previewVersion,
        prereleaseIdentifier,
    });
    pkgCommandsRan = applied.reduce((pkgs, result) => {
        pkgs[result.name].applied = result;
        return pkgs;
    }, pkgCommandsRan);
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands: versionCommands,
        commandPrefix: "post",
        command: "version",
        pkgCommandsRan,
        dryRun,
    });
    const { commands: publishCommands } = yield (0, assemble_1.mergeIntoConfig)({
        assembledChanges,
        config,
        command: "publish",
        cwd,
        dryRun,
        filterPackages,
        tag: branchTag,
    });
    if (publishCommands.length === 0) {
        console.log(`No commands configured to run on publish.`);
        return {
            response: `No commands configured to run on publish.`,
        };
    }
    const commandsToRun = yield (0, command_1.confirmCommandsToRun)({
        cwd,
        commands: publishCommands,
        command: "publish",
    });
    pkgCommandsRan = publishCommands.reduce((pkgs, pkg) => {
        pkgs[pkg.pkg] = {
            precommand: false,
            command: false,
            postcommand: false,
            pkg,
        };
        return pkgs;
    }, {});
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands: commandsToRun,
        commandPrefix: "pre",
        command: "publish",
        pkgCommandsRan,
        dryRun,
    });
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands: commandsToRun,
        command: "publish",
        pkgCommandsRan,
        dryRun,
    });
    return { commandsRan: pkgCommandsRan, pipeTemplate: publishCommands };
}
exports.preview = preview;
