"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.status = void 0;
const command_1 = require("@covector/command");
const files_1 = require("@covector/files");
const assemble_1 = require("@covector/assemble");
const apply_1 = require("@covector/apply");
const lodash_1 = require("lodash");
function* status({ command, dryRun = false, cwd = process.cwd(), filterPackages = [], modifyConfig = (c) => __awaiter(this, void 0, void 0, function* () { return c; }), branchTag = "", logs = true, }) {
    var _a;
    const config = yield modifyConfig(yield (0, files_1.configFile)({ cwd }));
    const pre = yield (0, files_1.readPreFile)({ cwd, changeFolder: config.changeFolder });
    const prereleaseIdentifier = !pre ? null : pre.tag;
    const changesPaths = yield (0, files_1.changeFiles)({
        cwd,
        changeFolder: config.changeFolder,
    });
    const changeFilesLoaded = yield (0, files_1.loadChangeFiles)({
        cwd,
        paths: changesPaths,
    });
    const assembledChanges = yield (0, assemble_1.assemble)({
        cwd,
        files: changeFilesLoaded,
        config,
        preMode: { on: !!pre, prevFiles: !pre ? [] : pre.changes },
    });
    if (changeFilesLoaded.length === 0) {
        if (logs)
            console.info("There are no changes.");
        const { commands: publishCommands } = yield (0, assemble_1.mergeIntoConfig)({
            assembledChanges,
            config,
            command: "publish",
            cwd,
            dryRun,
            filterPackages,
            tag: branchTag,
        });
        if (publishCommands.length === 0) {
            if (logs)
                console.log(`No commands configured to run on publish.`);
            return {
                response: `No commands configured to run on publish.`,
                pkgReadyToPublish: [],
            };
        }
        const commandsToRun = yield (0, command_1.confirmCommandsToRun)({
            cwd,
            commands: publishCommands,
            command: "publish",
        });
        if (commandsToRun.length > 0 && logs) {
            console.log(`There ${commandsToRun.length === 1
                ? `is 1 package`
                : `is ${commandsToRun.length} packages`} ready to publish which includes${commandsToRun.map((pkg) => { var _a; return ` ${pkg.pkg}@${(_a = pkg.pkgFile) === null || _a === void 0 ? void 0 : _a.version}`; })}`);
        }
        return {
            pkgReadyToPublish: commandsToRun,
            config,
            response: "No changes.",
        };
    }
    else if (!!pre && ((_a = assembledChanges === null || assembledChanges === void 0 ? void 0 : assembledChanges.changes) === null || _a === void 0 ? void 0 : _a.length) === 0) {
        if (logs) {
            console.info("There are no changes.");
            console.log("We have previously released the changes in these files:", changesPaths);
        }
        return { pkgReadyToPublish: [], response: "No changes." };
    }
    else {
        if (logs) {
            // write out all of the changes
            // TODO make it pretty
            console.log("changes:");
            Object.keys(assembledChanges.releases).forEach((release) => {
                console.log(`${release} => ${assembledChanges.releases[release].type}`);
                console.dir(assembledChanges.releases[release].changes, { depth: 4 });
            });
        }
        const allPackages = yield (0, files_1.readAllPkgFiles)({
            config,
            cwd,
        });
        const changes = (0, apply_1.changesConsideringParents)({
            assembledChanges,
            config,
            allPackages,
            prereleaseIdentifier,
        });
        const { commands, pipeTemplate, } = yield (0, assemble_1.mergeChangesToConfig)({
            assembledChanges: changes,
            config,
            command,
            dryRun,
            filterPackages,
            cwd,
        });
        // throws if failed validation
        yield (0, apply_1.validateApply)({
            //@ts-expect-error
            commands,
            // as the validate ends up mutating
            allPackages: (0, lodash_1.cloneDeep)(allPackages),
            prereleaseIdentifier,
        });
        const applied = yield (0, apply_1.apply)({
            //@ts-expect-error
            commands,
            config,
            allPackages,
            cwd,
            bump: false,
            prereleaseIdentifier,
            logs,
        });
        return {
            pkgVersion: commands,
            config,
            applied,
            pipeTemplate: pipeTemplate,
            response: `There are ${Object.keys(assembledChanges.releases).length} changes which include${Object.keys(assembledChanges.releases).map((release) => ` ${release} with ${assembledChanges.releases[release].type}`)}`,
        };
    }
}
exports.status = status;
