"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.version = void 0;
const command_1 = require("@covector/command");
const files_1 = require("@covector/files");
const assemble_1 = require("@covector/assemble");
const changelog_1 = require("@covector/changelog");
const apply_1 = require("@covector/apply");
function* version({ command, dryRun = false, cwd = process.cwd(), filterPackages = [], modifyConfig = (c) => __awaiter(this, void 0, void 0, function* () { return c; }), createContext, }) {
    const config = yield modifyConfig(yield (0, files_1.configFile)({ cwd }));
    const pre = yield (0, files_1.readPreFile)({ cwd, changeFolder: config.changeFolder });
    const prereleaseIdentifier = !pre ? null : pre.tag;
    const changesPaths = yield (0, files_1.changeFiles)({
        cwd,
        changeFolder: config.changeFolder,
    });
    const changeFilesLoaded = yield (0, files_1.loadChangeFiles)({
        cwd,
        paths: changesPaths,
    });
    const assembledChanges = yield (0, assemble_1.assemble)({
        cwd,
        files: changeFilesLoaded,
        config,
        preMode: { on: !!pre, prevFiles: !pre ? [] : pre.changes },
    });
    const allPackages = yield (0, files_1.readAllPkgFiles)({
        config,
        cwd,
    });
    yield (0, command_1.raceTime)({ t: config.timeout });
    const changes = (0, apply_1.changesConsideringParents)({
        assembledChanges,
        config,
        allPackages,
        prereleaseIdentifier,
    });
    const { commands, pipeTemplate, } = yield (0, assemble_1.mergeChangesToConfig)({
        assembledChanges: changes,
        config,
        command,
        dryRun,
        filterPackages,
        cwd,
    });
    if (dryRun) {
        console.log("==== commands ready to run ===");
        console.log(commands);
    }
    let pkgCommandsRan = Object.keys(config.packages).reduce((pkgs, pkg) => {
        pkgs[pkg] = {
            precommand: false,
            command: false,
            postcommand: false,
            applied: false,
        };
        return pkgs;
    }, {});
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands,
        commandPrefix: "pre",
        command,
        pkgCommandsRan,
        dryRun,
    });
    const applied = yield (0, apply_1.apply)({
        //@ts-expect-error
        commands,
        config,
        allPackages,
        cwd,
        bump: !dryRun,
        prereleaseIdentifier,
    });
    pkgCommandsRan = applied.reduce((pkgs, result) => {
        pkgs[result.name].applied = result;
        return pkgs;
    }, pkgCommandsRan);
    pkgCommandsRan = yield (0, changelog_1.fillChangelogs)({
        applied,
        //@ts-expect-error
        assembledChanges: changes,
        config,
        cwd,
        pkgCommandsRan,
        createContext,
        create: !dryRun,
    });
    pkgCommandsRan = yield (0, command_1.attemptCommands)({
        cwd,
        commands,
        commandPrefix: "post",
        command,
        pkgCommandsRan,
        dryRun,
    });
    if (command === "version" && !dryRun) {
        if (pre) {
            pre.changes = changesPaths;
            yield (0, files_1.writePreFile)({ preFile: pre, cwd });
        }
        else {
            yield (0, files_1.changeFilesRemove)({ cwd, paths: changesPaths });
        }
    }
    if (dryRun) {
        console.dir("==== result ===");
        console.dir(pkgCommandsRan);
    }
    return { commandsRan: pkgCommandsRan, pipeTemplate };
}
exports.version = version;
